Package: CAGEfightR
Type: Package
Title: Analysis of Cap Analysis of Gene Expression (CAGE) data using
        Bioconductor
Version: 1.31.0
Author: Malte Thodberg
Maintainer: Malte Thodberg <maltethodberg@gmail.com>
Description: CAGE is a widely used high throughput assay for measuring 
		transcription start site (TSS) activity. CAGEfightR is an R/Bioconductor 
		package for performing a wide range of common data analysis tasks for CAGE and 5'-end data in general. 
		Core functionality includes: import of CAGE TSSs (CTSSs), tag 
		(or unidirectional) clustering for TSS identification, 
		bidirectional clustering for enhancer identification, annotation with 
		transcript and gene models, correlation of TSS and enhancer expression, calculation of TSS shapes, quantification of 
		CAGE expression as expression matrices and genome brower visualization.
URL: https://github.com/MalteThodberg/CAGEfightR
BugReports: https://github.com/MalteThodberg/CAGEfightR/issues
Depends: R (>= 3.5), GenomicRanges (>= 1.61.1), rtracklayer (>=
        1.69.1), SummarizedExperiment (>= 1.39.1)
Imports: pryr(>= 0.1.3), assertthat(>= 0.2.0), methods(>= 3.6.3),
        Matrix(>= 1.2-12), BiocGenerics(>= 0.24.0), S4Vectors(>=
        0.16.0), IRanges(>= 2.12.0), Seqinfo, GenomicFeatures(>=
        1.61.4), GenomicAlignments(>= 1.45.1), BiocParallel(>= 1.12.0),
        GenomicFiles(>= 1.14.0), Gviz(>= 1.22.2), InteractionSet(>=
        1.9.4), GenomicInteractions(>= 1.15.1)
License: GPL-3 + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, BiocStyle, org.Mm.eg.db,
        TxDb.Mmusculus.UCSC.mm9.knownGene
VignetteBuilder: knitr
biocViews: Software, Transcription, Coverage, GeneExpression,
        GeneRegulation, PeakDetection, DataImport, DataRepresentation,
        Transcriptomics, Sequencing, Annotation, GenomeBrowsers,
        Normalization, Preprocessing, Visualization
git_url: https://git.bioconductor.org/packages/CAGEfightR
git_branch: devel
git_last_commit: c45462c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 21:37:07 UTC; biocbuild
