#' @importFrom methods setClass setGeneric setMethod setRefClass
NULL
#' @import assertthat BiocGenerics S4Vectors IRanges Seqinfo GenomicRanges
#'   rtracklayer GenomicFeatures SummarizedExperiment Gviz
NULL
